// -*- C++ -*-
/*!
 * @file  ImageDataConvert.cpp
 * @brief ImageDataConvert
 * @date  $Date$
 *
 * $Id$
 */

#include "ImageDataConvert.h"

cv::Mat GetMatImage(RTC::CameraImage ci)
{
    cv::Mat m_imageBuff;
    std::string format = (std::string)ci.format;
    if (format == "jpeg")
    {
        std::vector<uchar> buff;
        int len = ci.pixels.length();
        buff.resize(len);
        memcpy(&buff[0], &ci.pixels[0], sizeof(unsigned char) * len);
        cv::Mat jpegimage = cv::imdecode(cv::Mat(buff), CV_LOAD_IMAGE_COLOR);
        m_imageBuff = jpegimage;
    }
    else
    {
        if (ci.pixels.length() == ci.width * ci.height * 3)
        {
            m_imageBuff = cv::Mat::zeros(cvSize(ci.width, ci.height), CV_8UC3);
            memcpy(m_imageBuff.data, (void *)&(ci.pixels[0]), ci.pixels.length());
        }
        else if (ci.pixels.length() == ci.width * ci.height)
        {
            m_imageBuff = cv::Mat::zeros(cvSize(ci.width, ci.height), CV_8UC1);
            memcpy(m_imageBuff.data, (void *)&(ci.pixels[0]), ci.pixels.length());
        }
    }
    return m_imageBuff;
}

void SetCameraImage(RTC::CameraImage &ci, cv::Mat im, std::string strencode, int quality)
{
    ci.width = im.cols;
    ci.height = im.rows;

    ci.format = strencode.c_str();

    if (strencode == "jpeg")
    {
        std::vector<uchar> buff;
        std::vector<int> param = std::vector<int>(2);
        param[0] = CV_IMWRITE_JPEG_QUALITY;
        param[1] = 95;
        if (0 <= quality && quality <= 100)
        {
            param[1] = quality;
        }

        cv::imencode(".jpg", im, buff, param);

        int len = buff.size();
        ci.pixels.length(len);
        memcpy(&ci.pixels[0], &buff[0], sizeof(unsigned char) * len);
    }
    else
    {
        int len = im.channels() * im.rows * im.cols;
        ci.pixels.length(len);
        memcpy((void *)&(ci.pixels[0]), im.data, len);
    }
}
